/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */

import { randomInt } from "node:crypto";
import fastify, { type FastifyInstance } from "fastify";
import { html } from "./config/html.js";
import { mommy } from "./config/mommy.js";
import { phrases } from "./config/phrases.js";

/* eslint-disable @typescript-eslint/no-non-null-assertion -- Yeah, I'm not dealing with the logical branches. */

/**
 * Entry function. Exported solely for tests.
 * @returns The server instance for testing.
 */
const main = async(): Promise<FastifyInstance> => {
  const server = fastify({
    logger: false,
  });

  server.get("/", (_request, response) => {
    response.header("Content-Type", "text/html");
    response.send(html);
  });

  // @ts-expect-error Whatever.
  server.get("/api", (request: { query: { name?: string } }, response) => {
    const name = request.query.name ?? "dear";

    const phrase = phrases[randomInt(0, phrases.length)];
    const mom = mommy[randomInt(0, mommy.length)];

    const result = phrase!.
      replaceAll("{{ name }}", name).
      replaceAll("{{ mommy }}", mom!);
    response.header("Content-Type", "text/plain");
    response.send(result.toLowerCase());
  });

  await server.listen({ port: 8008 });

  return await server;
};

/* istanbul ignore if -- @preserve*/
if (process.env.TEST !== "true") {
  void main();
}

export { main };
